@@ -1,6 +1,6 @@
 /*
- *  Dvorak 7min 1.4, a Dvorak typing tutor
- *  Copyright (C) 1998-1999  Ragnar Hojland Espinosa
+ *  Dvorak 7min 1.5, a Dvorak typing tutor
+ *  Copyright (C) 1998-2000  Ragnar Hojland Espinosa
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,6 +33,10 @@
  *
  * Smoke/CRAP here - I just hacked 1.3 up and it's pretty late
  *                   a great excuse for messing up the code even more :)
+ *
+ * Hm, 1.5 and I'm pondering on sumbitting it to the Ofuscated C contest...
+ * It'll take a few versions more and some extra "features", and who knows.
+ * This baby is fun :)
  */
 
 #include <strings.h>
@@ -117,10 +121,10 @@ char colormap[] =
       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    3, -5, -5, -4, -4, -3, -3, -2, -2, 2, 2, 3, 4, 5, 
-      0, 0,0, 0,  0,
-   5, -5,-4,-3,-2,-2,2,2,3,4,5, 0,0, 
-      0,0, 0,
-   2, 0,0, -5,-5, -4,-3,-2,-2,2,2,3,4,5, 0,
+      0, 0, 0, 0,  0,
+   5, -5, -4, -3, -2, -2, 2, 2, 3, 4, 5, 0, 0, 
+      0, 0, 0,
+   2, 0, 0, -5, -5, -4, -3, -2, -2, 2, 2, 3, 4, 5, 0,
    0
 };
 
@@ -200,7 +204,7 @@ int importText (char const * const fileName, char** buf, unsigned int length)
    }
 #endif   
    
-   *buf = (char*) malloc (length+1);
+   *buf = (char*) malloc (length+1+1);
    if (!*buf) {
       postmortem = "memory allocation failed";
       fclose (file);
@@ -208,6 +212,7 @@ int importText (char const * const fileName, char** buf, unsigned int length)
    }
    
    column = 0;
+   (*buf[i++]) = '\x2';
    
    while (1) {
       c = fgetc (file);
@@ -350,6 +355,23 @@ void show_layout(char pressedKey, char pressed)
     refresh();
 }
 
+void myaddnstr (char const * text, int lenght)
+{
+   char *buf = malloc (lenght);  
+   char *obuf = buf;
+   int i = lenght;
+   
+   while (--i) {
+      if (*text != '\x2') {
+	 *(buf++) = *(text);
+      }
+      ++text;
+   }
+
+   addnstr (obuf, lenght);
+   free (obuf);
+}
+
 void do_text (char * const text)
 {
     int hits, misses;
@@ -372,7 +394,7 @@ void do_text (char * const text)
 
     move ((hideKeys ? 0 : 17), 0);
 
-    addnstr (text, this_page_size);  
+    myaddnstr (text, this_page_size);  
     hits = 0; misses = 0;
     refresh();
     noecho();
@@ -383,33 +405,46 @@ void do_text (char * const text)
 
     move (LINES - 1, 0);
     clrtoeol();
-    timeStart = time(0);
-    nextHelp = time(0) + (helpInterval / 1000);
+    timeStart = 0;
+    nextHelp = time(0) + (helpInterval / 1000);       
+   
     while (*p) {
-       int ch;
-        
+       int ch = 0;
+
+       if (*p == 0x2) {
+	  ++p;
+	  if (!timeStart) {
+	     timeStart = time(0);
+	  }
+       }
+       
        if (*p > 27) {
 	  show_layout(*p, PRESSED_REQUESTED);
        }
 
        do {
 	  ch = myGetch (*p);
-	  timeCurrent = time(0);
-	  speed = calcSpeed (timeStart, timeCurrent, hits);
-	  mvprintw (LINES - 1, 0, "CPS %.2f  CPM %.2f", speed, speed * 60);
-	  clrtoeol();
+	  if (timeStart) {
+	     timeCurrent = time(0);
+	     speed = calcSpeed (timeStart, timeCurrent, hits);
+	     mvprintw (LINES - 1, 0, "CPS %.2f  CPM %.2f", speed, speed * 60);	     
+	     clrtoeol();
+	  }
        } while (ch == ERR);
-
+       
        show_layout(ch, PRESSED_BY_USER);
         
         if (ch == *p || (*p == '\n' && ch == ' ')) {
             ++p;
             ++hits;
-	   /* any other possibilities? */
         } else {
+	    /* any other possibilities? */	   
             if ( (ch == 8 || ch == KEY_BACKSPACE || ch == KEY_DC || ch == 127)
                  && p > this_page_start) {
                 --p;
+	       if (*p == '\x2') {
+		  --p;
+	       }
             } else if (ch == 27 || ch == KEY_END) {
                 break;
             } else {
@@ -424,6 +459,9 @@ void do_text (char * const text)
                 if (nastiness && p > this_page_start) {
 		   --hits;
                    --p;
+		   if (*p == '\x2') {
+		      --p;
+		   } 	   
                 }
             }
         }
@@ -432,47 +470,63 @@ void do_text (char * const text)
         i = this_page_start;
         attron (A_BOLD);
         while (i < p) {
-            addch(*(i++));
+	   if (*i != 0x2) {
+	      addch(*i);
+	   }
+	   ++i;
         }
+        if (*i == '\x2') {
+	   ++i;
+	}
         attroff (A_BOLD);
         attron (A_REVERSE);
-        addch (*(i++));
+        if (*i != 0x2) {
+ 	   addch(*i);
+        }
+        ++i;       
         attroff (A_REVERSE);
         while (*i && i < this_page_start + this_page_size) {
-            addch(*(i++));
+	   if (*i != 0x2) {
+	      addch(*i);
+	   }
+	   ++i;
         }
         
         if (*p == 0x1) {
-            char *next;
-            ++p;
-            next = (char*) index (p, 0x1);
-            this_page_start = p;
-            this_page_size  = next ? next - this_page_start : strlen(this_page_start);
-	    move ((hideKeys ? 0 : 17), 0);
-            clrtobot();
-            addnstr (this_page_start, this_page_size);
-        }
+	   char *next;
+	   ++p;
+	   next = (char*) index (p, 0x1);
+	   this_page_start = p;
+	   this_page_size  = next ? next - this_page_start : strlen(this_page_start);
+	   move ((hideKeys ? 0 : 17), 0);
+	   clrtobot();
+	   myaddnstr (this_page_start, this_page_size);
+        } 
     }
 
     timeFinish = time(0);
     speed = calcSpeed(timeStart, timeFinish, hits);
     clear();
     cbreak();
-    mvprintw (0, 0, "Elapsed time: %d seconds", timeFinish - timeStart);
-    mvprintw (1, 0, "Total: %d  Misses: %d  Ratio: %.2f%%", hits, misses,
-              (float)100*(hits-misses) / hits);
-    mvprintw (2, 0, "CPS: %.2f  CPM: %.2f", speed, speed * 60);
-    mvprintw (4, 0, "[ ] Press ESCAPE to continue.");
-
-    move (4, 1);   
-    refresh();
+    if (!timeStart) {
+       mvprintw (0, 0, "You haven't done enough to get a good benchmark.");
+    } else {
+       mvprintw (0, 0, "Elapsed time: %d seconds", timeFinish - timeStart);
+       mvprintw (1, 0, "Total: %d  Misses: %d  Ratio: %.2f%%", hits, misses,
+		 (float)100*(hits-misses) / hits);
+       mvprintw (2, 0, "CPS: %.2f  CPM: %.2f", speed, speed * 60);
+    }
+   
+   mvprintw (4, 0, "[ ] Press ESCAPE to continue.");
+   move (4, 1);       
+   refresh();
     
-    while (getch() != 27);
+   while (getch() != 27);
     
-    curs_set (TRUE);
-    echo();
-    clear();
-    refresh(); 
+   curs_set (TRUE);
+   echo();
+   clear();
+   refresh(); 
 }
 
 void menuInteractive()
@@ -481,13 +535,13 @@ void menuInteractive()
     char buf[255];
     
     while (1) {
-        mvprintw (2, 0, "Dvorak7Min, Ragnar Hojland Espinosa, 1998-1999" );
+        mvprintw (2, 0, "Dvorak7Min, Ragnar Hojland Espinosa, 1998-2000" );
         mvprintw (3, 0, "enhanced by Smoke of CRAP, 1999");
         for (i = 0; lessons[i*2]; ++i) {
             mvprintw (i/2 + 5, (i%2) * 40, "%2d. %s", i+1, lessons[i*2]);
         }
         move (21, 0); clrtobot();
-        mvprintw (21, 0, "Type a lesson number ([N]astiness [Q]uit)? ");
+        mvprintw (21, 0, "Type a lesson number ([N]astiness [H]ide keyboard [Q]uit)? ");
         refresh();
         echo();
         getnstr (buf, sizeof(buf));
@@ -601,6 +655,7 @@ void showHelp()
 {
    printf("Usage: dvorak7min [OPTION]... [FILE]...\n\n"
 	  "  -b, --bell                   beep on error\n"
+          "  -f, --flash                  flash the screen on error\n"
 	  "  -u, --ugly                   use low ascii for artwork\n"
 	  "  -n, --nastiness              set nastiness on by default\n"
 	  "  -k, --hidekeys               hides keyboard layout\n"
@@ -611,8 +666,8 @@ void showHelp()
 
 void showVersionInfo()
 {
-    printf ("dvorak 7min tutor hack 1.3 - FREE Software with NO Warranty\n"
-            "(C) 1998-1999 Ragnar Hojland Espinosa <ragnar@lightside.ddns.org>\n"
+    printf ("dvorak 7min tutor hack 1.5 - FREE Software with NO Warranty\n"
+            "(C) 1998-2000 Ragnar Hojland Espinosa <ragnar@macula.net>\n"
             "Lessons are (C) 1995 Dan Wood <danwood@karelia.com>\n"
             "Enhancements in 1999 by Smoke of Crap aka Tijs van Bakel <smoke@casema.net>\n"
 	    "\n");
